#include <string>
#include <iostream>
#include <Windows.h>
#include "BayometricProxy.h"""

using namespace std;


string AppToken = "BayometricCPPSampleApplicationToken";

bool ShowGUI = true;
int Timeout = 30;

string sessionKey;

void CallCreateSession()
{
	
	AuthRequestInfo authRequestInfo;
	authRequestInfo.ShowGUI = ShowGUI ? 1 : 0;
	authRequestInfo.Timeout = Timeout;
	
	BFSAuthResponse response;

	response = CreateSession(authRequestInfo);

	if (response.Base.ResponseCode == 0) //Check for success. 
	{
		cout << "Session Created with SessionKey = " << response.SessionKey << endl;
		sessionKey = string(response.SessionKey);
	}
	else
	{
		cout << "Could not create session. Reason: " << response.Base.ReturnMessage << endl;
	}
}

void CallEndSession()
{
	BFSResponseBase response;

	response = EndSession(sessionKey);

	if (response.ResponseCode == 0) //Check for success. 
	{
		cout << "Session ended successfully" << endl;
		sessionKey = "";
	}
	else
	{
		cout << "Could not end session. Reason: " << response.ReturnMessage << endl;
	}
}

void CallRegisterPerson()
{

	string personID;
	cout << "Please type person ID: ";
	cin >> personID;
	 
	BFSResponse response;

	response = RegisterPerson(sessionKey,personID);

	if (response.Base.ResponseCode == 0) //Check for success. 
	{
		cout << "User registered with ID =  " << response.PersonID << endl;
	}
	else
	{
		cout << "Could register person. Reason: " << response.Base.ReturnMessage << endl;
	}
}

void CallUnregisterPerson()
{
	string personID;
	cout << "Please type person ID to be deleted: ";
	cin >> personID;

	BFSResponse response;

	response = UnregisterPerson(sessionKey, personID);

	if (response.Base.ResponseCode == 0) //Check for success. 
	{
		cout << "User removed successfully" << endl;
	}
	else
	{
		cout << "Could not unregister user. Reason: " << response.Base.ReturnMessage << endl;
	}
}

DWORD WINAPI CallSearch(LPVOID lpParameter)
{
	
	BFSResponse response;

	response = Search();

	if (response.Base.ResponseCode == 2) //Check for match. 
	{
		string personFound(response.PersonFoundID);
		cout << "Message: " << response.Base.ReturnMessage << " PersonFoundID: " << personFound << endl;
	}
	else if (response.Base.ResponseCode == 3)
	{
		cout << "User not found!" << endl;
	}
	else
	{
		cout << "Could not complete search. Reason: " << response.Base.ReturnMessage << endl;
	}

	return 0;
}

void CallCancelSearch()
{
	BFSResponse response;

	response = CancelSearch("");

	if (response.Base.ResponseCode == 0) //Check for success. 
	{
		cout << "Search canceled successfully" << endl;
	}
	else
	{
		cout << "Could not cancel search. Reason: " << response.Base.ReturnMessage << endl;
	}
}

void CallGetRegisteredPersons()
{
	BFSInfoResponse response;

	//Only listing 30 users
	response = GetRegisteredPersons(0, 30);

	if (response.Base.ResponseCode == 0 || response.Base.ResponseCode == 9) //Check for success or end of list. 
	{
		for (int i = 0; i < response.RegisteredPersons.size(); i++)
		{
			cout << response.RegisteredPersons[i] << endl;
		}
		
	}
	else
	{
		cout << "Could register person. Reason: " << response.Base.ReturnMessage << endl;
	}
}

void main()
{

	int opt = 0;
	bool showGUI = true;
	int timeout = 30;

	sessionKey = "";
	
	do
	{
		printf("\nSelection option below:\n");
		printf("1 - Create session\n");
		printf("2 - End session\n");
		printf("3 - Register person\n");
		printf("4 - Unregister person\n");
		printf("5 - Search\n");
		printf("6 - Cancel Search\n");
		printf("7 - Get registered users\n");
		printf("8 - Exit\n");
		
		cin >> opt;

		switch (opt)
		{
			case 1:
			{
					 CallCreateSession();
					 break;
			}
			case 2:
			{
					  if (sessionKey == "")
					  {
						  printf("Please create session first.\n");
					  }
					  else
					  {
						  CallEndSession();
					  }
					  break;
			}
			case 3:
			{
					  if (sessionKey == "")
					  {
						  printf("Please create session first.\n");
					  }
					  else
					  {
						  CallRegisterPerson();
					  }
					  break;
			}
			case 4:
			{
					  if (sessionKey == "")
					  {
						  printf("Please create session first.\n");
					  }
					  else
					  {
						  CallUnregisterPerson();
					  }
					  break;
			}
			case 5:
			{
					  
					  
					/*We need to run Search on different thread
					so that we are able to cancel it*/
					CreateThread(NULL, 0, CallSearch, NULL, 0, 0);
					 
					  break;
			}
			case 6:
			{
					 
					CallCancelSearch();
					break;
			}
			case 7:
			{
					  CallGetRegisteredPersons();
					  break;
			}
			default:
				break;
		}

	} while (opt != 8);

}